@@ -133,9 +133,19 @@ class Converter(object):
         return self._FORMAT_INFO
 
     def __init__(self, config, project_dir, output_tags=None, upload_dir=None, download_resources=True):
+        """ Initialize Label Studio Converter for Exports
+
+        :param config: string or dict: XML string with Label studio labeling config or path to this file or parsed_config
+        :param project_dir: upload root directory for images, audio and other labeling files
+        :param output_tags: it will be calculated automatically, contains label names
+        :param upload_dir: upload root directory with files that were imported using LS GUI
+        :param download_resources: if True, LS will try to download images, audio, etc and include them to export
+        """
         self.project_dir = project_dir
         self.upload_dir = upload_dir
         self.download_resources = download_resources
+        self._schema = None
+
         if isinstance(config, dict):
             self._schema = config
         elif isinstance(config, str):
@@ -146,6 +156,11 @@ class Converter(object):
                 config_string = config
             self._schema = parse_config(config_string)
 
+        if self._schema is None:
+            logger.warning('Label config or schema for Converter is not provided, '
+                           'it might be critical for some export formats, now set schema to empty dict')
+            self._schema = {}
+
         self._data_keys, self._output_tags = self._get_data_keys_and_output_tags(output_tags)
         self._supported_formats = self._get_supported_formats()
 
@@ -224,13 +239,13 @@ class Converter(object):
         if not ('Image' in input_tag_types and ('RectangleLabels' in output_tag_types or
                                                 'Rectangle' in output_tag_types and 'Labels' in output_tag_types)):
             all_formats.remove(Format.VOC.name)
-            all_formats.remove(Format.YOLO.name)
         if not ('Image' in input_tag_types and ('RectangleLabels' in output_tag_types or
                                                 'PolygonLabels' in output_tag_types) or
                                                 'Rectangle' in output_tag_types and 'Labels' in output_tag_types or
                                                 'PolygonLabels' in output_tag_types and 'Labels' in output_tag_types):
 
             all_formats.remove(Format.COCO.name)
+            all_formats.remove(Format.YOLO.name)
         if not ('Image' in input_tag_types and ('BrushLabels' in output_tag_types or 'brushlabels' in output_tag_types or
                                                 'Brush' in output_tag_types and 'Labels' in output_tag_types)):
             all_formats.remove(Format.BRUSH_TO_NUMPY.name)
@@ -328,7 +343,6 @@ class Converter(object):
             'lead_time': annotation.get('lead_time')
         }
 
-
     def _check_format(self, fmt):
         pass
 
@@ -387,10 +401,14 @@ class Converter(object):
 
     def convert_to_csv(self, input_data, output_dir, is_dir=True, **kwargs):
         self._check_format(Format.CSV)
-        ensure_dir(output_dir)
-        output_file = os.path.join(output_dir, 'result.csv')
+        if str(output_dir).endswith('.csv'):
+            output_file = output_dir
+        else:
+            ensure_dir(output_dir)
+            output_file = os.path.join(output_dir, 'result.csv')
         records = []
         item_iterator = self.iter_from_dir if is_dir else self.iter_from_json_file
+        keys = set()
 
         for item in item_iterator(input_data):
             record = deepcopy(item['input'])
@@ -410,27 +428,28 @@ class Converter(object):
             if 'agreement' in item:
                 record['agreement'] = item['agreement']
             records.append(record)
+            keys.update(list(record.keys()))
 
         # Previously we were using pandas dataframe to_csv() but that produced incorrect JSON so writing manually
-        with open(output_file, 'w') as outfile:
-            if kwargs['header']:
-                keys = records[0].keys()
-                outfile.write(kwargs['sep'].join(keys) + '\n')
-            for record in records:
-                line = []
-                for key in keys:
-                    if record[key] is None:
-                        line.append('')
-                    elif key == 'annotation_id':
-                        # Replicating previous implementation of converting None values to pandas.NA
-                        # which outputs in CSV files as an empty string
-                        if record[key] == None:
+        with open(output_file, 'w', encoding='utf8') as outfile:
+            if records:
+                if kwargs['header']:
+                    outfile.write(kwargs['sep'].join(keys) + '\n')
+                for record in records:
+                    line = []
+                    for key in keys:
+                        if record.get(key) is None:
                             line.append('')
+                        elif key == 'annotation_id':
+                            # Replicating previous implementation of converting None values to pandas.NA
+                            # which outputs in CSV files as an empty string
+                            if record[key] is None:
+                                line.append('')
+                            else:
+                                line.append(str(record[key]))
                         else:
                             line.append(str(record[key]))
-                    else:
-                        line.append(str(record[key]))
-                outfile.write(kwargs['sep'].join(line) + '\n')
+                    outfile.write(kwargs['sep'].join(line) + '\n')
 
     def convert_to_conll2003(self, input_data, output_dir, is_dir=True):
         self._check_format(Format.CONLL2003)
@@ -594,7 +613,24 @@ class Converter(object):
                 }
             }, fout, indent=2)
 
-    def convert_to_yolo(self, input_data, output_dir, output_image_dir=None, output_label_dir=None, is_dir=True):
+    def convert_to_yolo(self, input_data, output_dir, output_image_dir=None, output_label_dir=None, is_dir=True, split_labelers=False):
+        """Convert data in a specific format to the YOLO format.
+
+        Parameters
+        ----------
+        input_data : str
+            The input data, either a directory or a JSON file.
+        output_dir : str
+            The directory to store the output files in.
+        output_image_dir : str, optional
+            The directory to store the image files in. If not provided, it will default to a subdirectory called 'images' in output_dir.
+        output_label_dir : str, optional
+            The directory to store the label files in. If not provided, it will default to a subdirectory called 'labels' in output_dir.
+        is_dir : bool, optional
+            A boolean indicating whether `input_data` is a directory (True) or a JSON file (False).
+        split_labelers : bool, optional
+            A boolean indicating whether to create a dedicated subfolder for each labeler in the output label directory.
+        """
         self._check_format(Format.YOLO)
         ensure_dir(output_dir)
         notes_file = os.path.join(output_dir, 'notes.json')
@@ -625,8 +661,11 @@ class Converter(object):
                     logger.info('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
                     ), exc_info=True)
+            # create dedicated subfolder for each labeler if split_labelers=True 
+            labeler_subfolder = str(item['completed_by']) if split_labelers else ''
+            os.makedirs(os.path.join(output_label_dir, labeler_subfolder), exist_ok=True)
             # identify label file path
-            label_path = os.path.join(output_label_dir, os.path.splitext(os.path.basename(image_path))[0] + '.txt')
+            label_path = os.path.join(output_label_dir, labeler_subfolder, os.path.splitext(os.path.basename(image_path))[0] + '.txt')
             # Skip tasks without annotations
             if not item['output']:
                 logger.warning('No completions found for item #' + str(item_idx))
@@ -654,8 +693,8 @@ class Converter(object):
                 for key in ['rectanglelabels', 'polygonlabels', 'labels']:
                     if key in label and len(label[key]) > 0:
                         # change to save multi-label
-                        for item in label[key]:
-                            category_names.append(item)
+                        for category_name in label[key]:
+                            category_names.append(category_name)
 
                 if len(category_names) == 0:
                     logger.debug("Unknown label type or labels are empty: " + str(label))
@@ -674,6 +713,9 @@ class Converter(object):
                     if "rectanglelabels" in label or 'labels' in label:
                         x, y, w, h = self.rotated_rectangle(label)
                         annotations.append([category_id, (x + w / 2) / 100, (y + h / 2) / 100, w / 100, h / 100])
+                    elif "polygonlabels" in label:
+                        points_abs = [(x / 100, y / 100) for x, y in label["points"]]
+                        annotations.append([category_id] + [coord for point in points_abs for coord in point])
                     else:
                         raise ValueError(f"Unknown label type {label}")
             with open(label_path, 'w') as f:
